package cn.com.sailfish.linghang.filter;

import cn.com.sailfish.linghang.domain.User;
import cn.com.sailfish.linghang.repository.UserRepository;
import cn.com.sailfish.linghang.security.jwt.JWTConfigurer;
import cn.com.sailfish.linghang.security.jwt.TokenProvider;
import cn.com.sailfish.linghang.util.IPUtil;
import cn.com.sailfish.linghang.util.SecurityUtil;
import com.alibaba.fastjson.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.util.StringUtils;
import org.springframework.web.filter.GenericFilterBean;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;

import static cn.com.sailfish.linghang.common.ErrorConstants.*;
import static cn.com.sailfish.linghang.enumeration.UserInfoConstant.USER_TOKEN;

/**
 * Author: huangpeilin
 * Create at: 2018-12-12 11:50:48
 * Description: Filters incoming requests and installs a Spring Security principal if a header corresponding to a valid user is found.
 *
 * @author huangpeilin
 */
public class JWTFilter extends GenericFilterBean {

    private static Logger log = LoggerFactory.getLogger(JWTFilter.class);

    private TokenProvider tokenProvider;
    private UserRepository userRepository;

    public JWTFilter(TokenProvider tokenProvider, UserRepository userRepository) {
        this.tokenProvider = tokenProvider;
        this.userRepository = userRepository;
    }

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain)
            throws IOException, ServletException {
        HttpServletRequest httpServletRequest = (HttpServletRequest) servletRequest;
        String jwt = resolveToken(httpServletRequest);
        JSONObject responseObject = new JSONObject();
        if (StringUtils.hasText(jwt) && this.tokenProvider.validateToken(jwt)) {
            Authentication authentication = this.tokenProvider.getAuthentication(jwt);
            SecurityContextHolder.getContext().setAuthentication(authentication);
            // 校验token
            String card = null;
            String loginAccount = SecurityUtil.getCurrentUserLogin();
            User user = null;
            if (loginAccount != null){
                user = userRepository.findOneByCard(loginAccount).orElse(null);
                if (user == null) {
                    user = userRepository.findOneByMobilePhone(loginAccount).orElse(null);
                }
            }
            if (user == null) {
                responseObject.put("errorMessage", "user is not exists.");
                sendResult((HttpServletResponse) servletResponse, responseObject, USER_IS_NOT_EXISTS);
            }
            card = user.getCard();
            do {
                if (card == null){
                    break;
                }
                String url = getRequestUrl(httpServletRequest);
                if (url.startsWith("/apiv1/vr/")){
                    card = card + "-vr";
                }else{
                    card = card + "-web";
                }
                String oldToken = USER_TOKEN.get(card);
                if (oldToken == null){
                    break;
                }
                if (oldToken.equals(jwt)){
                    break;
                }
                responseObject.put("errorMessage", "user remote login.");
                sendResult((HttpServletResponse) servletResponse, responseObject, USER_REMOTE_LOGIN);
                return;
            }while (false);
            log.info(httpServletRequest.getMethod() + " " + httpServletRequest.getRequestURL()  + " " + IPUtil.getLoginIP(httpServletRequest) + " " + card);
            filterChain.doFilter(servletRequest, servletResponse);
        }else{
            filterChain.doFilter(servletRequest, servletResponse);
        }
    }

    private String resolveToken(HttpServletRequest request){
        String bearerToken = request.getHeader(JWTConfigurer.AUTHORIZATION_HEADER);
        if (StringUtils.hasText(bearerToken) && bearerToken.startsWith("Bearer ")) {
            return bearerToken.substring(7, bearerToken.length());
        }
        return null;
    }

    private void sendResult(HttpServletResponse servletResponse, JSONObject responseObject, int userRemoteLogin) throws IOException {
        responseObject.put("errorCode", userRemoteLogin);
        HttpServletResponse httpServletResponse = servletResponse;
        httpServletResponse.setHeader("Content-type","application/json;charset=UTF-8");
        httpServletResponse.setStatus(200);
        OutputStream stream = httpServletResponse.getOutputStream();
        stream.write(responseObject.toJSONString().getBytes("UTF-8"));
    }

    private String getRequestUrl(HttpServletRequest request){
        return request.getRequestURI();
    }
}
