package cn.com.sailfish.linghang.repository;

import cn.com.sailfish.linghang.domain.*;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.time.Instant;
import java.util.List;

/**
 * Author: AutoCoder (maintained by newma<newma@live.cn>)
 * Create: 2019-04-23 15:12:04
 *
 * @author huangpeilin
 */

@Repository
public interface ReportCardRepository extends JpaRepository<ReportCard, Long> {

    /**
     * 根据用户查找成绩单
     * @param card 用户工号
     * @param bIsDelete 是否在回收站中
     * @param sort
     * @return
     */
    List<ReportCard> findAllByCardAndBiosIsDelete(String card, boolean biosIsDelete, Sort sort);

    /**
     * 根据用户和考试的开启状态查找成绩单
     * @param card 用户工号
     * @param bIsDelete 是否在回收站中
     * @param switchStatus 考试的开启状态 0 开启 1 关闭
     * @return
     */
    List<ReportCard> findAllByCardAndBiosIsDeleteAndSwitchStatus(String card, boolean biosIsDelete,
                                                                 Integer switchStatus);

    /**
     * 根据用户和试卷查找成绩单
     * @param card 用户
     * @param questionPaper 试卷
     * @param sort
     * @return
     */
    List<ReportCard> findAllByCardAndQuestionPaper(String card, QuestionPaper questionPaper, Sort sort);

    /**
     * 根据考试ID，试卷ID，用户工号获取成绩单
     * @param examId 考试ID
     * @param questionPaperId 试卷ID
     * @param card  用户工号
     * @return
     */
    @Query(value = "select r from ReportCard r where r.exam.id = ?1 and r.questionPaper.id = ?2 and r.card = ?3")
    ReportCard findOneByExamIdAndQuestionPaperIdAndUserCard(Long examId, Long questionPaperId, String card);

    /**
     * 根据成绩单ID查找成绩单
     * @param id
     * @return
     */
    ReportCard findOneById(Long id);

    /**
     * 根据用户和考试完成状态分页获取成绩单
     * @param card 用户工号
     * @param status 考试完成状态
     * @param bIsDelete 是否在回收站中
     * @param pageable
     * @return
     */
    Page<ReportCard> findAllByCardAndCompletionStatusAndBiosIsDelete(String card, Integer status, Boolean biosIsDelete,
                                                                   Pageable pageable);

    /**
     * 根据用户和考试完成状态获取成绩单
     * @param card 用户工号
     * @param status 考试完成状态
     * @param bIsDelete 是否在回收站中
     * @return
     */
    List<ReportCard> findAllByCardAndCompletionStatusAndBiosIsDelete(String card, Integer status, Boolean biosIsDelete);

    /**
     * 根据用户，考试完成状态和试卷名字分页获取成绩单
     * @param card 用户工号
     * @param status 考试完成状态
     * @param bIsDelete 是否在回收站中
     * @param questionPaperName 试卷名字
     * @param pageable
     * @return
     */
    @Query("select r from ReportCard r where r.card = ?1 and r.completionStatus = ?2 and r.biosIsDelete = ?3 and r" +
            ".questionPaper.title like %?4%")
    Page<ReportCard> findAllByCradAndCompletionStatusAndBiosIsDeleteAndQuestionPaperTitleLike(String card,
                                                                                            Integer status,
                                                                                              boolean biosIsDelete,
                                                                                              String questionPaperName, Pageable pageable);

    /**
     * 根据用户，考试完成状态分页获取某个时间段的成绩单
     * @param card 用户工号
     * @param status 考试完成状态
     * @param bIsDelete 是否在回收站中
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query("select r from ReportCard r where r.card = ?1 and r.completionStatus = ?2 and r.biosIsDelete = ?3 and r" +
            ".examTime between ?4 and ?5")
    Page<ReportCard> findAllByCardAndCompletionStatusAndBiosIsDeleteAndExamTimeBetween(String card, Integer status,
                                                                                     boolean biosIsDelete,
                                                                                       Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据用户，考试完成状态和试卷名字分页获取某个时间段的成绩单
     * @param card 用户工号
     * @param status 考试完成状态
     * @param bIsDelete 是否在回收站中
     * @param questionPaperName 试卷名字
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param pageable
     * @return
     */
    @Query("select r from ReportCard r where r.card = ?1 and r.completionStatus = ?2 and r.biosIsDelete = ?3 and r" +
            ".questionPaper.title like %?4% and r.examTime between ?5 and ?6")
    Page<ReportCard> findAllByCardAndCompletionStatusAndBiosIsDeleteAndQuestionPaperTitleLikeAndExamTimeBetween(String card, Integer status, boolean biosIsDelete, String questionPaperName, Instant startTime, Instant endTime, Pageable pageable);

    /**
     * 根据考试获取班级成绩单
     * @param exam 考试
     * @param biosIsDelete 是否在回收站中 true 是 false 否
     * @return
     */
    List<ReportCard> findAllByExamAndBiosIsDelete(Exam exam, Boolean biosIsDelete);

    /**
     * 根据考试ID获取所有成绩单
     * @param exam
     * @return
     */
    List<ReportCard> findAllByExam(Exam exam);

    ReportCard findOneByExamAndCard(Exam exam, String card);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2")
    Page<ReportCard> findAllByAreaAndBiosIsDelete(Area area, Boolean biosIsDelete, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.exam.invigilator like " +
            "%?3% ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamInvigilatorLike(Area area, Boolean biosIsDelete,
                                                                      String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.exam.classType = ?3")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamClassType(Area area, Boolean biosIsDelete, String classType,
                                                                Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.exam.classType = ?3 and r" +
            ".exam.invigilator like %?4%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamClassTypeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String classType, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.questionPaper.title like " +
            "%?3%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndQuestionPaperTitleLike(Area area, Boolean biosIsDelete,
                                                                         String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.questionPaper.title like " +
            "%?3% and r.exam.invigilator like %?4%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.exam.classType = ?3 and r" +
            ".questionPaper.title like %?4%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamClassTypeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, String classType, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.exam.classType = ?3 and r" +
            ".questionPaper.title like %?4% and r.exam.invigilator like %?5%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamClassTypeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean bIsDelete, String classType, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.username like %?3%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndUsernameLike(Area area, Boolean biosIsDelete, String username,
                                                               Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.username like %?3% and r" +
            ".exam.invigilator like %?4%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndUsernameLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String username, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.username like %?3% and r" +
            ".exam.classType = ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndUsernameLikeAndExamClassType(
            Area area, Boolean biosIsDelete, String username, String classType, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.username like %?3% and r" +
            ".exam.classType = ?4 and r.exam.invigilator like %?5%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndUsernameLikeAndExamClassTypeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String username, String classType, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.username like %?3% and r.questionPaper.title like %?4%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndUsernameLikeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, String username, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.username like %?3% and r.questionPaper.title like %?4% and r.exam.invigilator like %?5%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndUsernameLikeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String username, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.username like %?3% and r.exam.classType = ?4 and r.questionPaper.title like %?5%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndUsernameLikeAndExamClassTypeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, String username, String classType, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.username like %?3% and r.exam.classType = ?4 " +
            "and r.questionPaper.title like %?5% and r.exam.invigilator like %?6%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndUsernameLikeAndExamClassTypeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String username, String classType, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLike(Area area, Boolean biosIsDelete, String card,
                                                           Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% and r.exam.invigilator like %?4%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% and r.exam.classType = ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamClassType(Area area, Boolean biosIsDelete,
                                                                           String card, String classType, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% and r.exam.classType = ?4 and r.exam.invigilator like %?5%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamClassTypeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, String classType, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% and r.questionPaper.title like %?4%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, String card, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% and r.questionPaper.title like %?4% and r.exam.invigilator like %?5%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% and r.exam.classType = ?4 and r.questionPaper.title like %?5%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamClassTypeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, String card, String classType, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% and r.exam.classType = ?4 " +
            "and r.questionPaper.title like %?5% and r.exam.invigilator like %?6%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamClassTypeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, String classType, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% and r.username like %?4%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndUsernameLike(Area area, Boolean biosIsDelete, String card,
                                                                          String username, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?4% and r.exam.invigilator like %?5%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndUsernameLikeAndExamInvigilatorLike(
            Area area, Boolean bIsDelete, String card, String username, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?4% and r.exam.classType = ?5")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndUsernameLikeAndExamClassType(
            Area area, Boolean biosIsDelete, String card, String username, String classType, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?4% and r.exam.classType = ?5 and r.exam.invigilator like %?6%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndUsernameLikeAndExamClassTypeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, String username, String classType, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?4% and r.questionPaper.title like %?5% and r.exam.invigilator like %?6%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndUsernameLikeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, String username, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?4% and r.questionPaper.title like %?5%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndUsernameLikeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, String card, String username, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?4% and r.exam.classType = ?5 and r.questionPaper.title like %?6%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndUsernameLikeAndExamClassTypeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, String card, String username, String classType, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?4% and r.exam.classType = ?5 and r.questionPaper.title like %?6% and r.exam.invigilator like %?7%")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndUsernameLikeAndExamClassTypeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, String username, String classType, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.examTime between ?4 and ?5")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetween(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.exam.invigilator like %?6% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.exam.classType = ?6 and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndExamClassType(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String classType, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.exam.classType = ?6 and r.exam.invigilator like %?7% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndExamClassTypeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String classType, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.questionPaper.title like %?6% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.questionPaper.title like %?6% and r.exam.invigilator like %?7% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.exam.classType = ?6 and r.questionPaper.title like %?7% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndExamClassTypeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String classType, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.exam.classType = ?6 and r.questionPaper.title like %?7% and r.exam.invigilator like %?8% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndExamClassTypeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String classType, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?6% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndUsernameLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String username, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?6% and r.exam.invigilator like %?7% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndUsernameLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String username, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?6% and r.exam.classType = ?7 and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndUsernameLikeAndExamClassType(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String username, String classType, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?6% and r.exam.classType = ?7 and r.exam.invigilator like %?8% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndiBiosIsDeleteAndCardLikeAndExamTimeBetweenAndUsernameLikeAndExamClassTypeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String username,
            String classType, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?6% and r.questionPaper.title like %?7% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndUsernameLikeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String username, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?6% and r.questionPaper.title like %?7% and r.exam.invigilator like %?8% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndUsernameLikeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String username, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?6% and r.exam.classType = ?7 and r.questionPaper.title like %?8% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndUsernameLikeAndExamClassTypeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String username, String classType, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.card like %?3% " +
            "and r.username like %?6% and r.exam.classType = ?7 and r.questionPaper.title like %?8% " +
            "and r.exam.invigilator like %?9% and r.examTime between ?4 and ?5 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndCardLikeAndExamTimeBetweenAndUsernameLikeAndExamClassTypeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, String card, Instant startTime, Instant endTime, String username, String classType,
            String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 and r.examTime between ?3 and ?4 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetween(Area area, Boolean biosIsDelete, Instant startTime,
                                                                  Instant endTime, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.exam.invigilator like %?5% and r.examTime between ?3 and ?4 ")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.exam.classType = ?5 and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndExamClassType(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String classType, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.exam.classType = ?5 and r.exam.invigilator like %?6% and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndExamClassTypeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String classType, String invigilator,
            Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.questionPaper.title like %?5% and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            " and r.questionPaper.title like %?5% and r.exam.invigilator like %?6% and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.exam.classType = ?5 and r.questionPaper.title like %?6% and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndExamClassTypeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String classType, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.exam.classType = ?5 and r.questionPaper.title like %?6% and r.exam.invigilator like %?7% and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndExamClassTypeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String classType, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.username like %?5% and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndUsernameLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String username, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.username like %?5% and r.exam.invigilator like %?6% and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndUsernameLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String username, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.username like %?5% and r.exam.classType = ?6 and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndUsernameLikeAndExamClassType(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String username, String classType, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.username like %?5% and r.exam.classType = ?6 and r.exam.invigilator like %?7% and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndUsernameLikeAndExamClassTypeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String username, String classType, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.username like %?5% and r.questionPaper.title like %?6% and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndUsernameLikeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String username, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.username like %?5% and r.questionPaper.title like %?6% and r.exam.invigilator like %?7% and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndUsernameLikeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String username, String questionPaperName, String invigilator, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.username like %?5% and r.exam.classType = ?6 and r.questionPaper.title like %?7% and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndUsernameLikeAndExamClassTypeAndQuestionPaperTitleLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String username, String classType, String questionPaperName, Pageable pageable);

    @Query("select r from ReportCard r where r.exam.area = ?1 and r.biosIsDelete = ?2 " +
            "and r.username like %?5% and r.exam.classType = ?6 " +
            "and r.questionPaper.title like %?7% and r.exam.invigilator like %?8% and r.examTime between ?3 and ?4")
    Page<ReportCard> findAllByAreaAndBiosIsDeleteAndExamTimeBetweenAndUsernameLikeAndExamClassTypeAndQuestionPaperTitleLikeAndExamInvigilatorLike(
            Area area, Boolean biosIsDelete, Instant startTime, Instant endTime, String username,
            String classType, String questionPaperName, String invigilator, Pageable pageable);

    ReportCard findOneByCardAndSwitchStatusAndCompletionStatus(String card, int switchStatus, int completionStatus);

    /**
     * 根据卡号获取成绩单
     * @param card
     * @return
     */
    List<ReportCard> findAllByCard(String card);
}
