package cn.com.sailfish.linghang.repository;

import cn.com.sailfish.linghang.domain.Area;
import cn.com.sailfish.linghang.domain.User;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Optional;

/**
 * Author: AutoCoder (maintained by newma<newma@live.cn>)
 * Create: 2019-04-23 15:12:04
 */

@Repository
public interface UserRepository extends JpaRepository<User, Long> {

    /**
     * 根据手机电话查找用户
     * @param mobilePhone
     * @return
     */
    Optional<User> findOneByMobilePhone(String mobilePhone);

    /**
     * 根据卡号查找用户
     * @param card
     * @return
     */
    Optional<User> findOneByCard(String card);

    /**
     * 根据电子邮箱查找用户
     * @param email
     * @return
     */
    Optional<User> findOneByEmail(String email);

    /**
     * 根据用户ID查找用户
     * @param id
     * @return
     */
    User findOneById(Long id);

    /**
     * 根据区域分页查找用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    Page<User> findAllByAreaAndActivationAndAuthorityGreaterThanEqual(Area area, boolean activation, Integer authority, Pageable pageable);

    /**
     * 根据区域和是否在考试队列分页查找用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param BiosIsTestQueue 是否在考试队列中
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean BiosIsTestQueue, Integer authority, Pageable pageable);

    /**
     * 根据区域和负责人名字（模糊）分页查找用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param leader 负责人名字 模糊查询
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.leader.username like %?3% and u.authority >= ?4")
    Page<User> findAllByAreaAndActivationAndLeaderUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String leader, Integer authority, Pageable pageable);

    /**
     * 根据区域和班别分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param classType 班别
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    Page<User> findAllByAreaAndActivationAndClassTypeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String classType, Integer authority, Pageable pageable);

    /**
     * 根据区域，班别，负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param classType 班别
     * @param leader 负责人名字
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.classType = ?3 and u.leader.username like %?4% and u.authority >= ?5")
    Page<User> findAllByAreaAndActivationAndClassTypeAndLeaderUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String classType, String leader, Integer authority, Pageable pageable);

    /**
     * 根据区域，用户名分业务查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param username 用户名 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.username like %?3% and u.authority >= ?4")
    Page<User> findAllByAreaAndActivationAndUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String username, Integer authority, Pageable pageable);

    /**
     * 根据区域，用户名和负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param username 用户名 模糊
     * @param leader 负责人名字 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.username like %?3% and u.leader.username like %?4% and u.authority >= ?5")
    Page<User> findAllByAreaAndActivationAndUsernameAndLeaderUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String username, String leader, Integer authority, Pageable pageable);

    /**
     * 根据区域，用户名和班别分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param classType 班别
     * @param username 用户名 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.classType = ?3 and u.username like %?4% and u.authority >= ?5")
    Page<User> findAllByAreaAndActivationAndClassTypeAndUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String classType, String username, Integer authority, Pageable pageable);

    /**
     * 根据区域，用户名，班别和负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param classType 班别
     * @param username 用户名 模糊
     * @param leader 负责人名字 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.classType = ?3 and u.username like %?4% and u.leader.username like %?5% and u.authority >= ?6")
    Page<User> findAllByAreaAndActivationAndClassTypeAndUsernameAndLeaderUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String classType, String username, String leader, Integer authority, Pageable pageable);

    /**
     * 根据区域，卡号分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.card like %?3% and u.authority >= ?4")
    Page<User> findAllByAreaAndActivationAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String card, Integer authority, Pageable pageable);

    /**
     * 根据区域，卡号和负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param card 卡号 模糊
     * @param leader 负责人名字 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.card like %?3% and u.leader.username like %?4% and u.authority >= ?5")
    Page<User> findAllByAreaAndActivationAndCardAndLeaderUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String card, String leader, Integer authority, Pageable pageable);

    /**
     * 根据区域，卡号和班别分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param classType 班别
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.classType = ?3 and u.card like %?4% and u.authority >= ?5")
    Page<User> findAllByAreaAndActivationAndClassTypeAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String classType, String card, Integer authority, Pageable pageable);

    /**
     * 根据区域，卡号，班别和负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param classType 班别
     * @param leader 负责人名字 模糊
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.classType = ?3 and u.leader.username like %?4% and u.card like %?5% " +
            "and u.authority >= ?6")
    Page<User> findAllByAreaAndActivationAndClassTypeAndLeaderUsernameLikeAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String classType, String leader, String card, Integer authority, Pageable pageable);

    /**
     * 根据区域，用户名，卡号分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param username 用户名 模糊
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.username like %?3% and u.card like %?4% and u.authority >= ?5")
    Page<User> findAllByAreaAndActivationAndUsernameLikeAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String username, String card, Integer authority, Pageable pageable);

    /**
     * 根据区域，用户名，负责人名字和卡号分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param username 用户名 模糊
     * @param leader 负责人名字 模糊
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.username like %?3% and u.leader.username like %?4% and u.card like %?5% " +
            "and u.authority >= ?6")
    Page<User> findAllByAreaAndActivationAndUsernameAndLeaderUsernameLikeAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String username, String leader, String card, Integer authority, Pageable pageable);

    /**
     * 根据区域，用户名和班别分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param classType 班别
     * @param username 用户名 模糊
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.classType = ?3 and u.username like %?4% and u.card like %?5% " +
            "and u.authority >= ?6")
    Page<User> findAllByAreaAndActivationAndClassTypeAndUsernameLikeAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String classType, String username, String card, Integer authority, Pageable pageable);

    /**
     * 根据区域，用户名，班别和负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param classType 班别
     * @param username 用户名 模糊
     * @param leader 负责人名字 模糊
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.classType = ?3 and u.username like %?4% and u.leader.username like %?5% " +
            "and u.card like %?6% and u.authority >= ?7")
    Page<User> findAllByAreaAndActivationAndClassTypeAndUsernameAndLeaderUsernameLikeAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String classType, String username, String leader, String card, Integer authority, Pageable pageable);

    /**
     * 根据区域和负责人名字（模糊）分页查找用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param leader 负责人名字 模糊查询
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.leader.username like %?4% and u.authority >= ?5")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndLeaderUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String leader, Integer authority,
            Pageable pageable);

    /**
     * 根据区域和班别分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String classType, Integer authority,
            Pageable pageable);

    /**
     * 根据区域，班别，负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param leader 负责人名字
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.classType = ?4 and u.leader.username like %?5% " +
            "and u.authority >= ?6")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndLeaderUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String classType, String leader, Integer authority,
            Pageable pageable);

    /**
     * 根据区域，用户名分业务查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param username 用户名 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.username like %?4% and u.authority >= ?5")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String username, Integer authority,
            Pageable pageable);

    /**
     * 根据区域，用户名和负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param username 用户名 模糊
     * @param leader 负责人名字 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.username like %?4% and u.leader.username like %?5% " +
            "and u.authority >= ?6")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndUsernameAndLeaderUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String username, String leader, Integer authority,
            Pageable pageable);

    /**
     * 根据区域，用户名和班别分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param username 用户名 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.classType = ?4 and u.username like %?5% and u.authority >= ?6")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String classType, String username,
            Integer authority, Pageable pageable);

    /**
     * 根据区域，用户名，班别和负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param username 用户名 模糊
     * @param leader 负责人名字 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.classType = ?4 and u.username like %?5% " +
            "and u.leader.username like %?6% and u.authority >= ?7")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndUsernameLikeAndLeaderUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String classType, String username, String leader,
            Integer authority, Pageable pageable);

    /**
     * 根据区域，卡号分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.card like %?4% and u.authority >= ?5")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String card, Integer authority, Pageable pageable);

    /**
     * 根据区域，卡号和负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param card 卡号 模糊
     * @param leader 负责人名字 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.card like %?4% and u.leader.username like %?5% " +
            "and u.authority >= ?6")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndCardAndLeaderUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String card, String leader, Integer authority,
            Pageable pageable);

    /**
     * 根据区域，卡号和班别分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.classType = ?4 and u.card like %?5% " +
            "and u.authority >= ?6")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String classType, String card, Integer authority,
            Pageable pageable);

    /**
     * 根据区域，卡号，班别和负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param leader 负责人名字 模糊
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.classType = ?4 and u.leader.username like %?5% " +
            "and u.card like %?6% and u.authority >= ?7")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndLeaderUsernameLikeAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String classType, String leader, String card,
            Integer authority, Pageable pageable);

    /**
     * 根据区域，用户名，卡号分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param username 用户名 模糊
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.username like %?4% and u.card like %?5% " +
            "and u.authority >= ?6")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndUsernameLikeAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String username, String card, Integer authority,
            Pageable pageable);

    /**
     * 根据区域，用户名，负责人名字和卡号分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param username 用户名 模糊
     * @param leader 负责人名字 模糊
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.username like %?4% and u.leader.username like %?5% " +
            "and u.card like %?6% and u.authority >= ?7")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndUsernameAndLeaderUsernameLikeAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String username, String leader, String card,
            Integer authority, Pageable pageable);

    /**
     * 根据区域，用户名和班别分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param username 用户名 模糊
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.classType = ?4 and u.username like %?5% " +
            "and u.card like %?6% and u.authority >= ?7")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndUsernameLikeAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String classType, String username, String card,
            Integer authority, Pageable pageable);

    /**
     * 根据区域，用户名，班别和负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param username 用户名 模糊
     * @param leader 负责人名字 模糊
     * @param card 卡号 模糊
     * @param authority 用户权限
     * @param pageable
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.classType = ?4 and u.username like %?5% " +
            "and u.leader.username like %?6% and u.card like %?7% and u.authority >= ?8")
    Page<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndUsernameLikeAndLeaderUsernameLikeAndCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String classType,
            String username, String leader, String card, Integer authority, Pageable pageable);

    /**
     * 根据用户ID和区域查找用户
     * @param id 用户ID
     * @param area 区域
     * @return
     */
    User findOneByIdAndArea(Long id, Area area);

    /**
     * 根据区域，用户名或者卡号查询所有用户
     * @param area 区域
     * @param activation  是否在回收站
     * @param column 用户名或者工号 模糊
     * @param authority 用户权限
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and (u.username like %?3% or u.card like %?3%) and u.authority >= ?4")
    List<User> findAllByAreaAndActivationAndUsernameLikeOrCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, String column, Integer authority);

    /**
     * 根据区域，用户类型，用户名或者卡号查询所有用户
     * @param area 区域
     * @param activation 是否在回收站中
     * @param type 用户类型 1 教员 0 学员
     * @param column 用户名或者工号 模糊
     * @param authority 用户权限
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.type = ?3 and (u.username like %?4% or u.card like %?4%) and u.authority >= ?5")
    List<User> findAllByAreaAndActivationAndTypeAndUsernameLikeOrCardLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, Integer type, String column, Integer authority);

    /**
     * 根据区域和是否在考试队列查找用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param authority 用户权限
     * @return
     */
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, Integer authority);

    /**
     * 根据区域和负责人名字（模糊）查找用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param leader 负责人名字 模糊查询
     * @param authority 用户权限
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.leader.username like %?4% and u.authority >= ?5")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndLeaderUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String leader, Integer authority);

    /**
     * 根据区域和班别查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param authority 用户权限
     * @return
     */
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String classType, Integer authority);

    /**
     * 根据区域，班别，负责人名字查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param leader 负责人名字
     * @param authority 用户权限
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 " +
            "and u.biosIsTestQueue = ?3 and u.classType = ?4 and u.leader.username like %?5% " +
            "and u.authority >= ?6")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndLeaderUsernameLikeAndAuthorityGreaterThanEqual(
            Area area, boolean activation, boolean biosIsTestQueue, String classType, String leader, Integer authority);

    /**
     * 根据区域，用户名查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param username 用户名 模糊
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.biosIsTestQueue = ?3 and u" +
            ".username like %?4%")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndUsernameLike(Area area, boolean activation,
                                                                         boolean biosIsTestQueue, String username);

    /**
     * 根据区域，用户名和负责人名字分页查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param username 用户名 模糊
     * @param leader 负责人名字 模糊
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.biosIsTestQueue = ?3 and u" +
            ".username like %?4% and u.leader.username like %?5%")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndUsernameAndLeaderUsernameLike(Area area,
                                                                                          boolean activation,
                                                                                            boolean biosIsTestQueue,
                                                                                            String username, String leader);

    /**
     * 根据区域，用户名和班别查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param username 用户名 模糊
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.biosIsTestQueue = ?3 and u" +
            ".classType = ?4 and u.username like %?5%")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndUsernameLike(Area area, boolean activation,
                                                                                    boolean biosIsTestQueue,
                                                                                    String classType, String username);

    /**
     * 根据区域，用户名，班别和负责人名字查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param username 用户名 模糊
     * @param leader 负责人名字 模糊
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.biosIsTestQueue = ?3 and u" +
            ".classType = ?4 and u.username like %?5% and u.leader.username like %?6%")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndUsernameLikeAndLeaderUsernameLike(Area area,
                                                                                                          boolean activation, boolean bIsTestQueue, String classType, String username, String leader);

    /**
     * 根据区域，卡号查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param card 卡号 模糊
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.biosIsTestQueue = ?3 and u" +
            ".card like %?4%")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndCardLike(Area area, boolean activation,
                                                                     boolean bIsTestQueue, String card);

    /**
     * 根据区域，卡号和负责人名字查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param card 卡号 模糊
     * @param leader 负责人名字 模糊
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.biosIsTestQueue = ?3 and u" +
            ".card like %?4% and u.leader.username like %?5%")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndCardAndLeaderUsernameLike(Area area, boolean activation,
                                                                                      boolean biosIsTestQueue,
                                                                                        String card, String leader);

    /**
     * 根据区域，卡号和班别查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param card 卡号 模糊
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.biosIsTestQueue = ?3 and u" +
            ".classType = ?4 and u.card like %?5%")
    List<User> findAllByAreaAndActivationAndBIsTestQueueAndClassTypeAndCardLike(Area area, boolean activation,
                                                                                boolean biosIsTestQueue,
                                                                                String classType, String card);

    /**
     * 根据区域，卡号，班别和负责人名字查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param leader 负责人名字 模糊
     * @param card 卡号 模糊
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.biosIsTestQueue = ?3 and u" +
            ".classType = ?4 and u.leader.username like %?5% and u.card like %?6%")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndLeaderUsernameLikeAndCardLike(Area area,
                                                                                                     boolean activation, boolean biosIsTestQueue, String classType, String leader, String card);

    /**
     * 根据区域，用户名，卡号查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param username 用户名 模糊
     * @param card 卡号 模糊
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.biosIsTestQueue = ?3 and u" +
            ".username like %?4% and u.card like %?5%")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndUsernameLikeAndCardLike(Area area, boolean activation,
                                                                                    boolean biosIsTestQueue,
                                                                                      String username, String card);

    /**
     * 根据区域，用户名，负责人名字和卡号查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param username 用户名 模糊
     * @param leader 负责人名字 模糊
     * @param card 卡号 模糊
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.biosIsTestQueue = ?3 and u" +
            ".username like %?4% and u.leader.username like %?5% and u.card like %?6%")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndUsernameAndLeaderUsernameLikeAndCardLike(Area area,
                                                                                                     boolean activation, boolean biosIsTestQueue, String username, String leader, String card);

    /**
     * 根据区域，用户名和班别查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param username 用户名 模糊
     * @param card 卡号 模糊
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.biosIsTestQueue = ?3 and u" +
            ".classType = ?4 and u.username like %?5% and u.card like %?6%")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndUsernameLikeAndCardLike(
            Area area, boolean activation, boolean biosIsTestQueue, String classType, String username, String card);

    /**
     * 根据区域，用户名，班别和负责人名字查询用户
     * @param area 区域
     * @param activation 是否在回收站
     * @param bIsTestQueue 是否在考试队列中
     * @param classType 班别
     * @param username 用户名 模糊
     * @param leader 负责人名字 模糊
     * @param card 卡号 模糊
     * @return
     */
    @Query(value = "select u from User u where u.area = ?1 and u.activation = ?2 and u.biosIsTestQueue = ?3 and u" +
            ".classType = ?4 and u.username like %?5% and u.leader.username like %?6% and u.card like %?7%")
    List<User> findAllByAreaAndActivationAndBiosIsTestQueueAndClassTypeAndUsernameLikeAndLeaderUsernameLikeAndCardLike(
            Area area, boolean activation, boolean biosIsTestQueue, String classType, String username, String leader,
            String card);

    /**
     * 根据用户名查找用户
     * @param username
     * @return
     */
    @Query(value = "select u from User u where u.username like %?1%")
    List<User> findAllByUsernameLike(String username);

    /**
     * 根据区域获取用户
     * @param area 区域
     * @param activation 是否在回收站
     * @return
     */
    List<User> findAllByAreaAndActivation(Area area, boolean activation);

    /**
     * 根据区域获取正在考试或者训练的所有用户
     * @param area 区域
     * @param statusList  状态集合
     * @param pageable
     * @return
     */
    Page<User> findAllByAreaAndStatusIn(Area area, List<Integer> statusList, Pageable pageable);

    /**
     * 根据区域获取用户
     * @param area
     * @return
     */
    List<User> findAllByArea(Area area);

    List<User> findAllByAreaAndBiosIsTestQueue(Area area, boolean biosIsTestQueue);

    /**
     * 根据用户权限查找用户
     * @param authority
     * @return
     */
    User findOneByAuthority(Integer authority);
}