package cn.com.sailfish.linghang.service;

import cn.com.sailfish.linghang.domain.FileInfo;
import cn.com.sailfish.linghang.exception.ServerErrorException;
import cn.com.sailfish.linghang.repository.FileInfoRepository;
import cn.com.sailfish.linghang.util.UUIDUtils;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.io.IOUtils;
import cn.com.sailfish.linghang.util.FileUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.time.Instant;
import java.util.HashMap;
import java.util.Map;

import static cn.com.sailfish.linghang.common.ErrorConstants.*;

/**
 * Author: huangpeilin
 * Create at: 2019-05-07 15:03:42
 * Description:
 *
 * @author huangpeilin
 */
@Service
public class FileService {

    public final Logger log = LoggerFactory.getLogger(FileService.class);

    private FileInfoRepository fileInfoRepository;
    private Map<String, String> imageTypeSet = new HashMap<>();

    @Autowired
    public FileService(FileInfoRepository fileInfoRepository) {
        this.fileInfoRepository = fileInfoRepository;
        this.imageTypeSet.put("jpg", "jpg");
        this.imageTypeSet.put("JPG", "JPG");
        this.imageTypeSet.put("png", "png");
        this.imageTypeSet.put("PNG", "PNG");
        this.imageTypeSet.put("bmp", "bmp");
        this.imageTypeSet.put("BMP", "BMP");
        this.imageTypeSet.put("jpeg", "jpeg");
        this.imageTypeSet.put("JPEG", "JPEG");
    }


    /**
     * 上传文件的地址
     */
    @Value("${sailfish.file.uploadAddress}")
    private String uploadAddress;

    /**
     * 图片上传最大为20M
     */
    private static final Long MAX_IMAGE_SIZE = Long.valueOf(20 * 1024);

    /**
     * 下载文件
     * @param key
     * @param response
     * @throws IOException
     */
    public void downloadFile(String key, HttpServletResponse response) throws IOException {
        FileInfo fileInfo = fileInfoRepository.findOneByKeyName(key);
        if (fileInfo != null){
            File downloadFile = new File(fileInfo.getPath());
            String fileName = fileInfo.getTitle();
            fileName = URLEncoder.encode(fileName, StandardCharsets.UTF_8.toString());
            fileName = fileName.replace("+", "%20");
            response.setContentType("application/octet-stream;charset=UTF-8");
            response.setHeader("Content-Disposition", "attachment; filename=" + fileName);
            FileInputStream inputStream = new FileInputStream(downloadFile);
            int len = inputStream.available();
            byte[] data = new byte[len];
            inputStream.read(data);
            inputStream.close();

            OutputStream out = new BufferedOutputStream(response.getOutputStream());
            out.write(data);
            out.flush();
            out.close();
        }
    }

    /**
     * 文件上传
     * @param file
     * @param key 文件的key,为null的时候创建，不为null的时候覆盖
     */
    public JSONObject uploadFile(MultipartFile file, String key) throws IOException, ServerErrorException {
        String fileName = file.getOriginalFilename();
        // 获取文件的后缀名
        String prefix = FileUtil.getExtensionName(fileName);
        // 检查文件后缀是否为图片后缀
        String type = imageTypeSet.get(prefix);
        if (type == null){
            log.info("file upload failed, file is not a image.");
            throw new ServerErrorException(FILE_IS_NOT_IMAGE);
        }
        // 获取文件的大小
        Long fileSize = file.getSize() / 1024;
        if (fileSize > MAX_IMAGE_SIZE){
            log.info("file upload failed, image more than 20MB.");
            throw new ServerErrorException(FILE_IS_TOO_BIG);
        }
        // 生成文件的key
        String keyName = getUUID();
        // 时间戳作为文件的命名
        String pathName = String.valueOf(Instant.now().toEpochMilli());
        String path = System.getProperty("user.dir") + uploadAddress;
        String filePath = getFilePath(pathName, path);
        // 文件记录
        FileInfo fileInfo = null;
        if (key != null) {
            fileInfo = fileInfoRepository.findOneByKeyName(key);
            if (fileInfo == null) {
                fileInfo = new FileInfo();
            } else {
                filePath = fileInfo.getPath();
                keyName = key;
            }
        } else {
            fileInfo = new FileInfo();
        }

        FileOutputStream fos = new FileOutputStream(filePath);
        InputStream is = file.getInputStream();
        IOUtils.copy(is, fos);
        // 上传文件完毕，保存文件的信息到数据库
        // 保存文件信息到数据库
        fileInfo = setMessageToFileInfo(fileInfo, keyName, filePath, fileName, prefix, fileSize);
        fileInfoRepository.save(fileInfo);
        JSONObject response = new JSONObject();
        response.put("key", keyName);
        response.put("size", fileSize);
        response.put("fileName", fileName);
        return response;
    }

    /**
     * 删除文件
     *
     * @param key
     */
    public void deleteFile(String key) throws ServerErrorException {
        if (key != null) {
            FileInfo fileInfo = fileInfoRepository.findOneByKeyName(key);
            if (fileInfo == null) {
                throw new ServerErrorException(FILE_IS_NOT_EXISTS);
            }
            File file = new File(fileInfo.getPath());
            if (file.isFile() && file.exists()) {
                System.gc();
                file.delete();
            }
            fileInfoRepository.delete(fileInfo);
        }
    }

    /**
     * 封装数据到fileInfo
     * @param fileInfo
     * @param keyName
     * @param filePath
     * @param fileName
     * @param prefix
     * @param fileSize
     * @return
     */
    private FileInfo setMessageToFileInfo(FileInfo fileInfo, String keyName, String filePath, String fileName, String prefix, Long fileSize) {
        if (fileInfo != null) {
            fileInfo.setKeyName(keyName);
            fileInfo.setPath(filePath);
            fileInfo.setTitle(fileName);
            fileInfo.setSize(fileSize);
            fileInfo.setType(prefix);
        }
        return fileInfo;
    }

    /**
     * 通过文件名字和硬盘存储地址构造文件路径
     * @param fileName
     * @param path
     * @return
     */
    private String getFilePath(String fileName, String path) {
        String filePath = path + "/" + fileName;
        File upload = new File(path);
        if (upload.exists() == false) {
            upload.mkdirs();
        }
        return filePath;
    }

    /**
     * 获取文件的Key
     *
     * @return
     */
    public String getUUID() {
        String key = UUIDUtils.getUUID();
        return key;
    }
}
