package cn.com.sailfish.linghang.service;

import cn.com.sailfish.linghang.common.ErrorConstants;
import cn.com.sailfish.linghang.exception.ServerErrorException;
import com.alibaba.fastjson.JSONObject;
import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.ExchangeTypes;
import org.springframework.beans.factory.annotation.Value;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeoutException;

/**
 * Author: huangpeilin
 * Create at: 2019-05-06 16:19:10
 * Description:
 *
 * @author huangpeilin
 */
@Slf4j
//@Service // todo add by 梁展鹏 20200228 暂时没有地方使用到，启动时候就不连接，加快启动速度
public class RabbitmqService {

    @Value("${spring.rabbitmq.host}")
    private String host;

    @Value("${spring.rabbitmq.port}")
    private int port;

    @Value("${spring.rabbitmq.username}")
    private String username;

    @Value("${spring.rabbitmq.password}")
    private String password;

    @Value("${sailfish.exchangeName}")
    private String exchangeName;

    private Connection connection;

    @PostConstruct
    public void createConnection() {
        connectToMQServer();
    }

    @PreDestroy
    public void closeConnection() {
        try {
            if (this.connection != null) {
                this.connection.close();
            }
        } catch (IOException e) {
            log.error(e.getMessage());
        }
    }

    private boolean connectToMQServer() {
        if (connection != null) {
            log.info("Already connected to MQ server");
            return true;
        }
        ConnectionFactory factory = new ConnectionFactory();
        factory.setHost(host);
        factory.setPort(port);
        factory.setUsername(username);
        factory.setPassword(password);
        log.info("rabbitmq Connection [{}]", host);

        try {
            this.connection = factory.newConnection();
            Channel channel = connection.createChannel();
            channel.exchangeDeclare(exchangeName, ExchangeTypes.FANOUT, true);
            channel.close();
            return true;
        } catch (IOException e) {
            e.printStackTrace();
        } catch (TimeoutException e) {
            e.printStackTrace();
        }
        return false;
    }

    private boolean sendDataToExchange(Channel channel, String exchangeName, byte[] data) throws ServerErrorException {
        try {
            declareExchange(channel, exchangeName);
            // 向指定exchange发送消息
            channel.basicPublish(exchangeName, "", null, data);
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new ServerErrorException(ErrorConstants.ERR_BAD_PARAM);
        }
        return true;
    }

    public boolean sendMessageToExchange(Channel channel, String exchangeName, String message) throws ServerErrorException {
        byte[] data = getMessageBytes(message);
        return this.sendDataToExchange(channel, exchangeName, data);
    }

    private byte[] getMessageBytes(String message) throws ServerErrorException {
        byte[] data;
        try {
            data = message.getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            log.error(e.getMessage());
            throw new ServerErrorException(ErrorConstants.GET_BYTE_ERROR);
        }
        return data;
    }

    private boolean sendDataToQueue(Channel channel, String queueName, byte[] data) throws ServerErrorException {
        try {
            // 向指定queue发送消息
            channel.basicPublish("", queueName, null, data);
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new ServerErrorException(ErrorConstants.ERR_BAD_PARAM);
        }
        return true;
    }

    public boolean sendMessageToQueue(Channel channel, String queueName, String message) throws ServerErrorException {
        byte[] data = getMessageBytes(message);
        return this.sendDataToQueue(channel, queueName, data);
    }

    public Channel createChannel() throws IOException {
        return connection.createChannel();
    }

    public void closeChannel(Channel channel) throws IOException, TimeoutException {
        channel.close();
    }

    /**
     * 绑定
     *
     * @param exchangeName
     * @param queueName
     * @throws ServerErrorException
     */
    public void bind(Channel channel, String exchangeName, String queueName) throws ServerErrorException {
        try {
            //declareExchange(channel, exchangeName);
            // 队列与交易所绑定
            channel.queueBind(queueName, exchangeName, "");
        } catch (IOException e) {
            e.printStackTrace();
            throw new ServerErrorException(ErrorConstants.ERR_BAD_PARAM);
        }
    }

    /**
     * 解绑
     *
     * @param exchangeName
     * @param queueName
     * @throws ServerErrorException
     */
    public void unbind(Channel channel, String exchangeName, String queueName) throws ServerErrorException {
        try {
            // 队列与交易所绑定
            channel.queueUnbind(queueName, exchangeName, "");
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new ServerErrorException(ErrorConstants.ERR_BAD_PARAM);
        }
    }

    /**
     * 声明并清空queue中的消息
     *
     * @param queueName
     */
    public void declareAndPurgeQueue(Channel channel, String queueName) throws ServerErrorException {
        try {
            boolean durable = true;
            // 声明队列  持久化的队列
            channel.queueDeclare(queueName, durable, false, false, null);
            // 清空队列中的消息数据
            channel.queuePurge(queueName);
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new ServerErrorException(ErrorConstants.ERR_BAD_PARAM);
        }
    }

    /**
     * 声明queue中的消息
     *
     * @param queueName
     */
    public void declareQueue(Channel channel, String queueName) throws ServerErrorException {
        try {
            boolean durable = true;
            // 声明队列  持久化的队列
            channel.queueDeclare(queueName, durable, false, false, null);
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new ServerErrorException(ErrorConstants.ERR_BAD_PARAM);
        }
    }

    /**
     * 声明exchange
     *
     * @param exchangeName
     */
    public void declareExchange(Channel channel, String exchangeName) throws ServerErrorException {
        // 声明exchange 交换所
        // ExchangeTypes.FANOUT  所有发送到Fanout Exchange的消息都会被转发到与该Exchange 绑定(Binding)的所有Queue上。
        // ExchangeTypes.DIRECT  所有发送到Direct Exchange的消息被转发到RouteKey中指定的Queue。
        // ExchangeTypes.TOPIC   所有发送到Topic Exchange的消息被转发到所有关心RouteKey中指定Topic的Queue上.
        try {
            channel.exchangeDeclare(exchangeName, ExchangeTypes.FANOUT, true);
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new ServerErrorException(ErrorConstants.ERR_BAD_PARAM);
        }
    }

    /**
     * 向指定消息队列发送考试开始结束消息
     *
     * @param channel
     * @param type
     * @param examMessage
     * @throws ServerErrorException
     */
    public void sendExamMessage(Channel channel, String type, String examMessage) throws ServerErrorException {
        JSONObject message = new JSONObject();
        message.put("type", type);
        message.put("data", examMessage);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("msgType", "event");
        jsonObject.put("message", message.toJSONString());
        sendMessageToExchange(channel, exchangeName, jsonObject.toJSONString());
    }

    /**
     * 发送用户被强制登录的消息
     *
     * @param queueName
     * @param type      跳转类型
     * @throws ServerErrorException
     */
    public void sendLoginMessage(Channel channel, String queueName, String type) throws ServerErrorException {
        List<Object> arguments = new ArrayList<>();
        JSONObject message = new JSONObject();
        message.put("type", type);
        message.put("arguments", arguments);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("msgType", "command");
        jsonObject.put("message", message.toJSONString());
        declareQueue(channel, queueName);
        sendMessageToQueue(channel, queueName, jsonObject.toJSONString());
    }

}
