package cn.com.sailfish.linghang.util;

import cn.com.sailfish.linghang.common.ErrorMessages;
import cn.com.sailfish.linghang.exception.ServerErrorException;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.Base64Utils;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.net.URLEncoder;

import static cn.com.sailfish.linghang.common.ErrorConstants.FILE_IS_NOT_THE_EXCEL;
import static cn.com.sailfish.linghang.common.ErrorConstants.IMPORT_INFO_IS_FAIL;

/**
 * Author: huangpeilin
 * Create at: 2019-05-07 15:11:33
 * Description:
 *
 * @author huangpeilin
 */
public class FileUtil {

    public static final Logger log = LoggerFactory.getLogger(FileUtil.class);

    /**
     * 获取文件的扩展名
     *
     * @param filename
     * @return
     */
    public static String getExtensionName(String filename) {
        if ((filename != null) && (filename.length() > 0)) {
            int dot = filename.lastIndexOf('.');
            if ((dot > -1) && (dot < (filename.length() - 1))) {
                return filename.substring(dot + 1);
            }
        }
        return filename;
    }

    /**
     * 获取不带扩展名的文件名
     *
     * @param filename
     * @return
     */
    public static String getFileNameNoEx(String filename) {
        if ((filename != null) && (filename.length() > 0)) {
            int dot = filename.lastIndexOf('.');
            if ((dot > -1) && (dot < (filename.length()))) {
                return filename.substring(0, dot);
            }
        }
        return filename;
    }

    /**
     * 下载文件时，针对不同浏览器，进行附件名的编码
     *
     * @param filename 下载文件名
     * @param agent    客户端浏览器
     * @return 编码后的下载附件名
     * @throws IOException
     */
    public static String encodeDownloadFilename(String filename, String agent)
            throws IOException {
        // 火狐浏览器
        if (agent.contains("Firefox")) {
            filename = "=?UTF-8?B?"
                    + Base64Utils.encode(filename.getBytes("utf-8"))
                    + "?=";
            filename = filename.replaceAll("\r\n", "");
        } else { // IE及其他浏览器
            filename = URLEncoder.encode(filename, "utf-8");
            filename = filename.replace("+", " ");
        }
        return filename;
    }

    /**
     * 检查文件是不是excel文件，并将文件的内容装化为workbook内容
     *
     * @param file
     * @return
     * @throws ServerErrorException
     */
    public static Workbook checkFileIsExcel(MultipartFile file) throws ServerErrorException {
        String fileName = file.getOriginalFilename();
        Workbook workbook;
        try {
            if (fileName.endsWith("xlsx")) {
                workbook = new XSSFWorkbook(file.getInputStream());
            } else if (fileName.endsWith("xls")) {
                workbook = new HSSFWorkbook(file.getInputStream());
            } else {
                log.error("File is not the excel.");
                throw new ServerErrorException(FILE_IS_NOT_THE_EXCEL);
            }
        } catch (IOException e) {
            log.error("Import info is fail.");
            throw new ServerErrorException(IMPORT_INFO_IS_FAIL);
        }
        return workbook;
    }

    /**
     * 判断行中是空格的个数
     *
     * @param row
     * @return
     */
    public static int rowEmpty(Row row) {
        int rowEmpty = 0;
        for (int i = 0; i <= row.getPhysicalNumberOfCells(); i++) {
            Cell cell = row.getCell(i);
            if (cell == null || "".equals(cell.toString().trim())) {
                rowEmpty++;
            }
        }
        return rowEmpty;
    }

    /**
     * 打印出导入数据后的错误信息
     *
     * @param sheet     excel文件中的工作表
     * @param row       工作表中的行
     * @param errorCode 错误码
     * @author wuchongliang
     */
    public static String printErrorMessage(int sheet, Row row, int errorCode) {
        return "The " + (row.getRowNum() + 1) + " line of the " + sheet + " sheet," + ErrorMessages.getErrorMessage(errorCode);
    }
}
